@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteapp.c,v 1.71 2006/01/15 21:09:41 gpastore Exp $"
+#ident "$Id: vteapp.c,v 1.77 2006/03/15 11:02:59 behdad Exp $"
 
 #include "../config.h"
 
@@ -34,13 +34,7 @@
 #include "debug.h"
 #include "vte.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
-#else
-#define _(String) String
-#define bindtextdomain(package,dir)
-#endif
+#include <glib/gi18n-lib.h>
 
 #define DINGUS1 "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?"
 #define DINGUS2 "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]"
@@ -117,12 +111,20 @@ destroy_and_quit(GtkWidget *widget, gpointer data)
 static void
 destroy_and_quit_eof(GtkWidget *widget, gpointer data)
 {
-	g_print("Detected EOF.\n");
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC)) {
+		g_print("Detected EOF.\n");
+	}
+#endif
 }
 static void
 destroy_and_quit_exited(GtkWidget *widget, gpointer data)
 {
-	g_print("Detected child exit.\n");
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC)) {
+		g_print("Detected child exit.\n");
+	}
+#endif
 	destroy_and_quit(widget, data);
 }
 
@@ -392,7 +394,11 @@ int
 main(int argc, char **argv)
 {
 	GtkWidget *window, *hbox, *scrollbar, *widget;
-	char *env_add[] = {"FOO=BAR", "BOO=BIZ", NULL};
+	char *env_add[] = {
+#ifdef VTE_DEBUG
+		"FOO=BAR", "BOO=BIZ",
+#endif
+		NULL};
 	const char *background = NULL;
 	gboolean transparent = FALSE, audible = TRUE, blink = TRUE,
 		 debug = FALSE, dingus = FALSE, geometry = TRUE, dbuffer = TRUE,
@@ -670,7 +676,7 @@ main(int argc, char **argv)
 
 	/* Set the default font. */
 	if (font == NULL) {
-		font = "Sans 12";
+		font = "Monospace 12";
 	}
 	vte_terminal_set_font_from_string_full(VTE_TERMINAL(widget),
 					       font, antialias);
@@ -744,10 +750,12 @@ main(int argc, char **argv)
 						  command, NULL, env_add,
 						  working_directory,
 						  TRUE, TRUE, TRUE);
+	#ifdef VTE_DEBUG
 			if (command == NULL) {
 				vte_terminal_feed_child(VTE_TERMINAL(widget),
 							"pwd\n", -1);
 			}
+	#endif
 		} else {
 			long i;
 			i = vte_terminal_forkpty(VTE_TERMINAL(widget),
