@@ -16,21 +16,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtebg.c,v 1.2 2003/05/01 20:50:54 nalin Exp $"
+#ident "$Id: vtebg.c,v 1.11 2006/03/15 11:02:59 behdad Exp $"
 #include "../config.h"
+#include <stdio.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include "debug.h"
 #include "marshal.h"
 #include "vtebg.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
-#else
-#define _(String) String
-#define bindtextdomain(package,dir)
-#endif
+#include <glib/gi18n-lib.h>
 
 struct VteBgPrivate {
 	GList *cache;
@@ -39,7 +34,10 @@ struct VteBgPrivate {
 static VteBg *singleton_bg = NULL;
 static void vte_bg_set_root_pixmap(VteBg *bg, GdkPixmap *pixmap);
 static void vte_bg_init(VteBg *bg, gpointer *klass);
+static GdkPixbuf *_vte_bg_resize_pixbuf(GdkPixbuf *pixbuf,
+					gint min_width, gint min_height);
 
+#if 0
 static const char *
 vte_bg_source_name(enum VteBgSourceType type)
 {
@@ -59,6 +57,7 @@ vte_bg_source_name(enum VteBgSourceType type)
 	}
 	return "unknown";
 }
+#endif
 
 #ifndef X_DISPLAY_MISSING
 
@@ -91,6 +90,28 @@ vte_bg_native_new(GdkWindow *window)
 	return pvt;
 }
 
+static void
+_vte_bg_display_sync(VteBg *bg)
+{
+#if GTK_CHECK_VERSION(2,2,0)
+	gdk_display_sync(gdk_drawable_get_display(bg->native->window));
+#else
+	XSync(GDK_DISPLAY(), FALSE);
+#endif
+}
+
+static gboolean
+_vte_property_get_pixmaps(GdkWindow *window, GdkAtom atom,
+			  GdkAtom *type, int *size,
+			  XID **pixmaps)
+{
+	return gdk_property_get(window, atom, GDK_TARGET_PIXMAP,
+				0, INT_MAX - 3,
+				FALSE,
+				type, NULL, size,
+				(guchar**) pixmaps);
+}
+
 static GdkPixmap *
 vte_bg_root_pixmap(VteBg *bg)
 {
@@ -102,16 +123,9 @@ vte_bg_root_pixmap(VteBg *bg)
 	pixmap = NULL;
 	pixmaps = NULL;
 	gdk_error_trap_push();
-	if (gdk_property_get(bg->native->window,
-			     bg->native->atom,
-			     GDK_TARGET_PIXMAP,
-			     0,
-			     INT_MAX,
-			     FALSE,
-			     &prop_type,
-			     NULL,
-			     &prop_size,
-			     (guchar**) &pixmaps)) {
+	if (_vte_property_get_pixmaps(bg->native->window, bg->native->atom,
+				      &prop_type, &prop_size,
+				      &pixmaps)) {
 		if ((prop_type == GDK_TARGET_PIXMAP) &&
 		    (prop_size >= sizeof(XID) &&
 		    (pixmaps != NULL))) {
@@ -119,12 +133,23 @@ vte_bg_root_pixmap(VteBg *bg)
 			pixmap = gdk_pixmap_foreign_new_for_display(gdk_drawable_get_display(bg->native->window), pixmaps[0]);
 #else
 			pixmap = gdk_pixmap_foreign_new(pixmaps[0]);
+#endif
+#ifdef VTE_DEBUG
+			if (_vte_debug_on(VTE_DEBUG_MISC) ||
+			    _vte_debug_on(VTE_DEBUG_EVENTS)) {
+				gint pwidth, pheight;
+				gdk_drawable_get_size(pixmap,
+						      &pwidth, &pheight);
+				fprintf(stderr, "New background image %dx%d\n",
+					pwidth, pheight);
+			}
 #endif
 		}
 		if (pixmaps != NULL) {
 			g_free(pixmaps);
 		}
 	}
+	_vte_bg_display_sync(bg);
 	gdk_error_trap_pop();
 	return pixmap;
 }
@@ -166,10 +191,8 @@ vte_bg_root_filter(GdkXEvent *xevent, GdkEvent *event, gpointer data)
 {
 	return GDK_FILTER_CONTINUE;
 }
-
 #endif
 
-
 static void
 vte_bg_class_init(VteBgClass *klass, gpointer data)
 {
@@ -261,10 +284,10 @@ vte_bg_init(VteBg *bg, gpointer *klass)
 /* Generate lookup tables for desaturating an image toward a given color.  The
  * saturation value is a floating point number between 0 and 1. */
 static void
-vte_bg_generate_desat_tables(const GdkColor *color, double saturation,
-			     guchar red[256],
-			     guchar green[256],
-			     guchar blue[256])
+_vte_bg_generate_desat_tables(const GdkColor *color, double saturation,
+			      guchar red[256],
+			      guchar green[256],
+			      guchar blue[256])
 {
 	int i;
 	/* Zero saturation -> exactly match the tinting color. */
@@ -292,7 +315,7 @@ vte_bg_generate_desat_tables(const GdkColor *color, double saturation,
 				 (saturation * i),
 				 0, 255);
 		blue[i] = CLAMP(((1.0 - saturation) * (color->blue >> 8)) +
-			        (saturation * i),
+				(saturation * i),
 				0, 255);
 	}
 }
@@ -357,6 +380,62 @@ vte_bg_cache_prune(VteBg *bg)
 	vte_bg_cache_prune_int(bg, FALSE);
 }
 
+/**
+ * _vte_bg_resize_pixbuf:
+ * @pixmap: a #GdkPixbuf, or NULL
+ * @min_width: the requested minimum_width
+ * @min_height: the requested minimum_height
+ *
+ * The background pixbuf may be tiled, and if it is tiled, it may be very, very
+ * small.  This function creates a pixbuf consisting of the passed-in pixbuf
+ * tiled to a usable size.
+ *
+ * Returns: a new #GdkPixbuf, unrefs @pixbuf.
+ */
+static GdkPixbuf *
+_vte_bg_resize_pixbuf(GdkPixbuf *pixbuf, gint min_width, gint min_height)
+{
+	GdkPixbuf *tmp;
+	gint src_width, src_height;
+	gint dst_width, dst_height;
+	gint x, y;
+
+	if (!GDK_IS_PIXBUF(pixbuf)) {
+		return pixbuf;
+	}
+
+	src_width = gdk_pixbuf_get_width(pixbuf);
+	src_height = gdk_pixbuf_get_height(pixbuf);
+	dst_width = (((min_width - 1) / src_width) + 1) * src_width;
+	dst_height = (((min_height - 1) / src_height) + 1) * src_height;
+	if ((dst_width == src_width) && (dst_height == src_height)) {
+		return pixbuf;
+	}
+
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC) || _vte_debug_on(VTE_DEBUG_EVENTS)) {
+		fprintf(stderr, "Resizing (root?) pixbuf from %dx%d to %dx%d\n",
+			src_width, src_height, dst_width, dst_height);
+	}
+#endif
+
+	tmp = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf),
+			     gdk_pixbuf_get_has_alpha(pixbuf),
+			     gdk_pixbuf_get_bits_per_sample(pixbuf),
+			     dst_width, dst_height);
+	for (y = 0; y < dst_height; y += src_height) {
+		for (x = 0; x < dst_width; x += src_width) {
+			gdk_pixbuf_copy_area(pixbuf,
+					     0, 0, src_width, src_height,
+					     tmp,
+					     x, y);
+		}
+	}
+
+	g_object_unref(G_OBJECT(pixbuf));
+	return tmp;
+}
+
 static void
 vte_bg_set_root_pixmap(VteBg *bg, GdkPixmap *pixmap)
 {
@@ -398,7 +477,7 @@ vte_bg_desaturate_pixbuf(GdkPixbuf *pixbuf,
 	long stride, width, height, channels, x, y;
 	guchar *pixels;
 
-	vte_bg_generate_desat_tables(tint, saturation, red, green, blue);
+	_vte_bg_generate_desat_tables(tint, saturation, red, green, blue);
 
 	stride = gdk_pixbuf_get_rowstride(pixbuf);
 	width = gdk_pixbuf_get_width(pixbuf);
@@ -495,7 +574,6 @@ vte_bg_get_pixmap(VteBg *bg,
 	GdkBitmap *mask;
 	GdkPixbuf *pixbuf;
 	char *file;
-	int width, height;
 
 	if (bg == NULL) {
 		bg = vte_bg_get();
@@ -527,14 +605,35 @@ vte_bg_get_pixmap(VteBg *bg,
 	switch (source_type) {
 	case VTE_BG_SOURCE_ROOT:
 		if (GDK_IS_PIXMAP(bg->root_pixmap)) {
-			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+			int width, height;
+			/* Tell GTK+ that this foreign pixmap shares the
+			 * root window's colormap. */
 			rcolormap = gdk_drawable_get_colormap(gdk_get_default_root_window());
-			pixbuf = gdk_pixbuf_get_from_drawable(NULL,
-							      bg->root_pixmap,
-							      rcolormap,
-							      0, 0,
-							      0, 0,
-							      width, height);
+			if (gdk_drawable_get_colormap(bg->root_pixmap) == NULL) {
+				gdk_drawable_set_colormap(bg->root_pixmap,
+							  rcolormap);
+			}
+
+			/* Retrieve the pixmap's size. */
+			gdk_error_trap_push();
+			width = height = -1;
+			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+			_vte_bg_display_sync(bg);
+			gdk_error_trap_pop();
+
+			/* If the pixmap gave us a valid size, retrieve its
+			 * contents. */
+			if ((width > 0) && (height > 0)) {
+				gdk_error_trap_push();
+				pixbuf = gdk_pixbuf_get_from_drawable(NULL,
+								      bg->root_pixmap,
+								      NULL,
+								      0, 0,
+								      0, 0,
+								      width, height);
+				_vte_bg_display_sync(bg);
+				gdk_error_trap_pop();
+			}
 		}
 		break;
 	case VTE_BG_SOURCE_PIXBUF:
@@ -569,6 +668,11 @@ vte_bg_get_pixmap(VteBg *bg,
 	pixmap = NULL;
 	mask = NULL;
 	if (GDK_IS_PIXBUF(pixbuf)) {
+		/* If the image is smaller than 256x256 then tile it into a
+		 * pixbuf that is at least this large.  This is done because
+		 * tiling a 1x1 pixmap onto the screen using thousands of calls
+		 * to XCopyArea is very slow. */
+		pixbuf = _vte_bg_resize_pixbuf(pixbuf, 256, 256);
 		gdk_pixbuf_render_pixmap_and_mask_for_colormap(pixbuf,
 							       colormap,
 							       &pixmap, &mask,
@@ -597,6 +701,7 @@ vte_bg_get_pixbuf(VteBg *bg,
 	struct VteBgCacheItem *item;
 	GObject *cached;
 	GdkPixbuf *pixbuf;
+	GdkColormap *rcolormap;
 	char *file;
 
 	if (bg == NULL) {
@@ -628,16 +733,35 @@ vte_bg_get_pixbuf(VteBg *bg,
 	switch (source_type) {
 	case VTE_BG_SOURCE_ROOT:
 		if (GDK_IS_PIXMAP(bg->root_pixmap)) {
-			GdkColormap *colormap;
 			gint width, height;
+
+			/* If the pixmap doesn't have a colormap, tell GTK+ that
+			 * it shares the root window's colormap. */
+			rcolormap = gdk_drawable_get_colormap(gdk_get_default_root_window());
+			if (gdk_drawable_get_colormap(bg->root_pixmap) == NULL) {
+				gdk_drawable_set_colormap(bg->root_pixmap, rcolormap);
+			}
+
+			/* Read the pixmap's size. */
+			gdk_error_trap_push();
+			width = height = -1;
 			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
-			colormap = gdk_drawable_get_colormap(bg->root_pixmap);
-			pixbuf = gdk_pixbuf_get_from_drawable(NULL,
-							      bg->root_pixmap,
-							      colormap,
-							      0, 0,
-							      0, 0,
-							      width, height);
+			_vte_bg_display_sync(bg);
+			gdk_error_trap_pop();
+
+			/* If we got a valid size, read the pixmap's
+			 * contents. */
+			if ((width > 0) && (height > 0)) {
+				gdk_error_trap_push();
+				pixbuf = gdk_pixbuf_get_from_drawable(NULL,
+								      bg->root_pixmap,
+								      NULL,
+								      0, 0,
+								      0, 0,
+								      width, height);
+				_vte_bg_display_sync(bg);
+				gdk_error_trap_pop();
+			}
 		}
 		break;
 	case VTE_BG_SOURCE_PIXBUF:
